﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MotorBoat.Entities;
using MotorBoat;
using MotorBoat.MovementStrategy;

namespace MotorBoat.DrawningObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningBoat
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBoat? EntityBoat { get; protected set; }

        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth = 800;

        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight = 450;

        /// <summary>
        /// Левая координата прорисовки 
        /// </summary>
        protected int _startPosX;

        /// <summary>
        /// Верхняя кооридната прорисовки 
        /// </summary>
        protected int _startPosY;

        /// <summary>
        /// Ширина прорисовки 
        /// </summary>
        protected readonly int _boatWeight = 165;

        /// <summary>
        /// Высота прорисовки 
        /// </summary>
        protected readonly int _boatHeight = 80;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, 
        ///нельзя создать объект в этих размерах</returns>
        public DrawningBoat(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _boatWeight || height < _boatHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningCar
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawningObjectBoat(this);

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param> 
        /// <param name="boatWidth">Ширина прорисовки автомобиля</param>
        /// <param name="boatHeight">Высота прорисовки автомобиля</param>
        protected DrawningBoat(int speed, double weight, Color bodyColor, int width, int height, int boatWidth, int boatHeight)
        {
            if (width <= _boatWeight || height <= _boatHeight)
                return;
            _pictureWidth = width;
            _pictureHeight = height;
            _boatWeight = boatWidth;
            _boatHeight = boatHeight;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (x >= 0 && x + _boatWeight <= _pictureWidth && y >= 0 && y + _boatHeight <= _pictureHeight)
            {
                _startPosX = x;
                _startPosY = y;
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBoat == null)
            {
                return;
            }
            Pen pen = new(Color.Black, 1);

            Brush mainBrush = new SolidBrush(EntityBoat.BodyColor);

            // корпус
            Point[] hull = new Point[]
            {
                new Point(_startPosX + 5, _startPosY + 0),
                new Point(_startPosX + 120, _startPosY + 0),
                new Point(_startPosX + 160, _startPosY + 35),
                new Point(_startPosX + 120, _startPosY + 70),
                new Point(_startPosX + 5, _startPosY + 70),
            };
            g.FillPolygon(mainBrush, hull);
            g.DrawPolygon(pen, hull);

            // осн часть
            Brush blockBrush = new SolidBrush(Color.Olive);
            g.FillRectangle(blockBrush, _startPosX + 20, _startPosY + 15, 80, 40);
            g.DrawRectangle(pen, _startPosX + 20, _startPosY + 15, 80, 40);
        }
        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _boatWeight;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _boatHeight;
        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityBoat == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityBoat.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityBoat.Step > 0,
                //вправо
                DirectionType.Right => _startPosX + EntityBoat.Step + _boatWeight < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + EntityBoat.Step + _boatHeight < _pictureHeight,
                _ => false,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityBoat == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityBoat.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityBoat.Step;
                    break;
                //вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityBoat.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityBoat.Step;
                    break;
            }
        }
    }
}